-----------------------------------------------------------------------------------------------
--[[Client Lua Script for ConfigWindow - By Ciuine
	
	When ConfigWindow (CW) initializes it will send out the event, "ConfigWindow_InitReq".
	
	Your add-on should register this event to a function that uses;
	Event_FireGenericEvent("ConfigWindow_InitRep", {name = "YourAdd-onNameHere"})
	This will add the add-on to the list to display in the selection grid.
	
	When your add-on is selected via the selection grid, CW will send out the event, "ConfigWindow_BtnReq_YourAdd-onNameHere".
	
	Your add-on must then reply with Event_FireGenericEvent("ConfigWindow_BtnRes", tConfig) where tConfig is a sortable table of desired display.
	An example: 
	{
	[1] = {type = "radial", val = true, text = "Random Radial Button"};
	[2] = {type = "text", text = "This is a slider that does stuff:"};
	[3] = {type = "slider", min = .5, max = 1.5, tick = .1, val = 1};
	}
	
	Requirements:
		Top level table must be sortable for order continuity and recall.
		Each item must contain a "type" key and should not contain a "num" key.
		
		radial: val = boolean, text = string
		text: text = string
		slider: min, max, tick, val = float
		header: text = string
		divider: nothing required
		color: val = table or ApolloColor.new
		drop down: val, text = string, grid = table of string
		edit line: val, text = string
		
	Optional:
		Any key not listed will be sent through unharmed. Recommend "var" or "name" for tracking.
		A drop down button as a primary button allows for the ability to generate simplified tabled duplicate configurations by firing the BtnRes event after the drop down selection.
		Say for large combat text, damage meter, or hud configurations.
		
		Event_FireGenericEvent("ConfigWindow_DemReq", {name = "YourAdd-onNameHere"}) is a demand request. It will make CW show for your add-on.
		
	User Input:
		When an action is selected CW will send out "ConfigWindow_CfgUpd_YourAdd-onNameHere" with a table of data with the modifications the user made.
	
--  GPL3]]
-----------------------------------------------------------------------------------------------
 
require "Window"
require "Apollo"
 
-----------------------------------------------------------------------------------------------
-- ConfigWindow Module Definition
-----------------------------------------------------------------------------------------------

local ConfigWindow = {}
 
-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------

function ConfigWindow:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    return o
end

function ConfigWindow:Init()
	local bHasConfigureFunction = true
	local strConfigureButtonText = "ConfigWindow"
	local tDependencies = {
	}
    Apollo.RegisterAddon(self, bHasConfigureButton, strConfigureButtonText, tDependencies)
end
 
-----------------------------------------------------------------------------------------------
-- ConfigWindow OnLoad
-----------------------------------------------------------------------------------------------

function ConfigWindow:OnLoad()
	self.xmlDoc = XmlDoc.CreateFromFile("ConfigWindow.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
	self.wndMain = Apollo.LoadForm(self.xmlDoc, "ConfigWindow", nil, self)		
	    self.wndMain:Show(false, true)
		self.wndCfgCont = self.wndMain:FindChild("CfgCont")
		self.tAddonList = {}
		self.strCurrentAddon = ""
		self.bIGC = false
		self.wndColorPicker = Apollo.LoadForm(self.xmlDoc, "ColorPicker", nil, self)
		self.wndColorDisplay = self.wndColorPicker:FindChild("ColorDisplay")
		self.wndRedSlider = self.wndColorPicker:FindChild("RedSlider")
		self.wndGreenSlider = self.wndColorPicker:FindChild("GreenSlider")
		self.wndBlueSlider = self.wndColorPicker:FindChild("BlueSlider")
		self.wndAlphaSlider = self.wndColorPicker:FindChild("AlphaSlider")
end

-----------------------------------------------------------------------------------------------
-- ConfigWindow OnDocLoaded
-----------------------------------------------------------------------------------------------

function ConfigWindow:OnDocLoaded()

		Apollo.RegisterEventHandler("ConfigWindow_InitRep", "OnCWInitReply", self)
		Apollo.RegisterEventHandler("ConfigWindow_BtnRes", "OnCWBtnRes", self)
		Apollo.RegisterEventHandler("ConfigWindow_DemReq", "OnCWDemReq", self)
		
		Apollo.RegisterSlashCommand("cw", "OnCWShow", self)
		Event_FireGenericEvent("ConfigWindow_InitReq")
end

-----------------------------------------------------------------------------------------------
-- ConfigWindow Functions
-----------------------------------------------------------------------------------------------

function ConfigWindow:OnCWInitReply(p)
	table.insert(self.tAddonList, p.name)
end

function ConfigWindow:OnCWBtnRes(p)
	if type(p) == "table" then
		local function SortByOrder(a, b) return a:GetData().num < b:GetData().num end
		self.wndCfgCont:DestroyChildren()
		self.bIGC = false
		for i, v in pairs(p) do
			if v.type then
				v.num = i
				ConfigWindow:CreateNextItem(v, self)
			end
		end
		self.wndCfgCont:ArrangeChildrenVert(0, SortByOrder)
	end
end

function ConfigWindow:OnCWDemReq(p)
	self.strCurrentAddon = p.name
	self.wndMain:Show(true)
	self.wndMain:FindChild("DropDown"):SetText(self.strCurrentAddon)
	Event_FireGenericEvent("ConfigWindow_BtnReq_" .. self.strCurrentAddon)
end

function ConfigWindow:CreateNextItem(tItem, self)
	if tItem.type == "radial" then
		local wndItem = Apollo.LoadForm(self.xmlDoc, "RadBtn", self.wndCfgCont, self)
		wndItem:Show(true)
		wndItem:SetText(tItem.text)
		wndItem:FindChild("Radial"):SetCheck(tItem.val)
		wndItem:SetData(tItem)
	elseif tItem.type == "slider" then
		local wndItem = Apollo.LoadForm(self.xmlDoc, "Slider", self.wndCfgCont, self)
		wndItem:Show(true)
		wndItem:SetMinMax(tItem.min, tItem.max, tItem.tick)
		wndItem:SetValue(tItem.val)
		wndItem:SetData(tItem)
	elseif tItem.type == "text" then
		local wndItem = Apollo.LoadForm(self.xmlDoc, "ConText", self.wndCfgCont, ConfigWindow)
		wndItem:Show(true)
		wndItem:SetText(tItem.text)
		wndItem:SetData(tItem)
	elseif tItem.type == "header" then
		local wndItem = Apollo.LoadForm(self.xmlDoc, "HeadText", self.wndCfgCont, self)
		wndItem:Show(true)
		wndItem:SetText(tItem.text)
		wndItem:SetData(tItem)
	elseif tItem.type == "divider" then
		local wndItem = Apollo.LoadForm(self.xmlDoc, "Div", self.wndCfgCont, self)
		wndItem:Show(true)
		wndItem:SetData(tItem)
	elseif tItem.type == "edit line" then
		local wndItem = Apollo.LoadForm(self.xmlDoc, "EditLine", self.wndCfgCont, self)
		wndItem:Show(true)
		wndItem:SetText(tItem.text)
		wndItem:FindChild("EditBox"):SetText(tItem.val)
		wndItem:SetData(tItem)
	elseif tItem.type == "color" then
		local wndItem = Apollo.LoadForm(self.xmlDoc, "ColBtn", self.wndCfgCont, self)
		wndItem:Show(true)
		wndItem:SetText(tItem.text)
		wndItem:FindChild("Color"):SetBGColor(tItem.val)
		wndItem:SetData(tItem)
		self.bIGC = true
		self.wndColorPicker:Show(true)
	elseif tItem.type == "drop down" then
		local wndItem = Apollo.LoadForm(self.xmlDoc, "DropDown", self.wndCfgCont, self)
		wndItem:Show(true)
		wndItem:FindChild("DDText"):SetText(tItem.text)
		wndItem:FindChild("DDBtn"):SetText(tItem.val)
		wndItem:SetData(tItem)
	end
end

function ConfigWindow:OnCWShow()
	self.wndMain:Show(not self.wndMain:IsVisible())
	self.wndColorPicker:Show(self.bIGC)
end

function ConfigWindow:OnConfigure()
	self.wndMain:Show(true)
	self.wndColorPicker:Show(self.bIGC)
end

function ConfigWindow:OnClose()
	self.wndMain:Show(false)
	self.wndColorPicker:Show(false)
end



---------------------------------------------------------------------------------------------------
-- Form Functions
---------------------------------------------------------------------------------------------------

function ConfigWindow:ADropDown( wndHandler, wndControl )
	local strBtnName = wndControl:GetData()
	table.sort(self.tAddonList, function (a, b) return a < b end)
	wndControl:GetParent():FindChild("ADDGrid"):Show(true)		
	wndControl:GetParent():FindChild("ADDGrid"):DeleteAll()
	for i, v in pairs(self.tAddonList) do
		if strBtnName == nil or strBtnName.name == nil or strBtnName.name ~= v then
			wndControl:GetParent():FindChild("ADDGrid"):AddRow(v, nil, v)
		end
	end
end

function ConfigWindow:DropDown( wndHandler, wndControl )
	local tBtnData = wndControl:GetParent():GetData()
	local strBtnName = wndControl:GetText()
	wndControl:GetParent():FindChild("DDGrid"):DeleteAll()
	for i, v in pairs(tBtnData.grid) do
		if strBtnName ~= v or strBtnName == nil then
			wndControl:GetParent():FindChild("DDGrid"):AddRow(v, nil, v)
		end
	end
	wndControl:GetParent():FindChild("DDGrid"):Show(not wndControl:GetParent():FindChild("DDGrid"):IsVisible())
end

function ConfigWindow:RadChange( wndHandler, wndControl )
	local tData = wndControl:GetParent():GetData()
	tData.val = wndControl:IsChecked()
	Event_FireGenericEvent("ConfigWindow_CfgUpd_" .. self.strCurrentAddon, tData)
end

function ConfigWindow:SlideUpdate( wndHandler, wndControl, fValue, fOldValue )
	local tData = wndControl:GetData()
	tData.val = fValue
	Event_FireGenericEvent("ConfigWindow_CfgUpd_" .. self.strCurrentAddon, tData)
end

function ConfigWindow:EditBoxReturn( wndHandler, wndControl )
	local tData = wndControl:GetParent():GetData()
	tData.val = wndControl:GetText()
	Event_FireGenericEvent("ConfigWindow_CfgUpd_" .. self.strCurrentAddon, tData)
end

function ConfigWindow:ColorPick( wndHandler, wndControl )
	local udCurColor = self.wndColorDisplay:GetBGColor()
	wndControl:SetBGColor(udCurColor)
	local tData = wndControl:GetParent():GetData()
	--if type(tData.val) == "table" then
		--tData.val = CColor:ToTable(udCurColor)
	--else
		tData.val = udCurColor
	--end
	Event_FireGenericEvent("ConfigWindow_CfgUpd_" .. self.strCurrentAddon, tData)
end

function ConfigWindow:ColorChange(wndHandler, wndControl, fValue, fOldValue)
	local rColor = self.wndRedSlider:GetValue() / 255
	local gColor = self.wndGreenSlider:GetValue() / 255
	local bColor = self.wndBlueSlider:GetValue() / 255
	local aColor = self.wndAlphaSlider:GetValue() / 255
	self.wndColorDisplay:SetBGColor({r = rColor, g = gColor, b = bColor, a = aColor})
end

function ConfigWindow:DDSelChange(wndHandler, wndControl, nRow)
	wndControl:Show(false)
	local tData = wndControl:GetParent():GetData() 
	tData.val = wndControl:GetCellText(nRow, 1)
	wndControl:GetParent():FindChild("DDBtn"):SetText(tData.val)
	Event_FireGenericEvent("ConfigWindow_CfgUpd_" .. self.strCurrentAddon, tData)
end

function ConfigWindow:DDAddonChange(wndHandler, wndControl, nRow)
	wndControl:Show(false)
	local tData = wndControl:GetCellText(nRow, 1)
	self.strCurrentAddon = tData
	wndControl:GetParent():FindChild("DropDown"):SetText(tData)
	Event_FireGenericEvent("ConfigWindow_BtnReq_" .. tData)
end

-----------------------------------------------------------------------------------------------
-- ConfigWindow Instance
-----------------------------------------------------------------------------------------------

local ConfigWindowInst = ConfigWindow:new()
ConfigWindowInst:Init()
